@@ -49,6 +49,7 @@ class MultiTypeExperiment(Experiment):
         description: Optional[str] = None,
         is_test: bool = False,
         experiment_type: Optional[str] = None,
+        properties: Optional[Dict[str, Any]] = None,
     ) -> None:
         """Inits Experiment.
 
@@ -64,6 +65,7 @@ class MultiTypeExperiment(Experiment):
             description: Description of the experiment.
             is_test: Convenience metadata tracker for the user to mark test experiments.
             experiment_type: The class of experiments this one belongs to.
+            properties: Dictionary of this experiment's properties.
         """
 
         self._default_trial_type = default_trial_type
@@ -91,6 +93,7 @@ class MultiTypeExperiment(Experiment):
             description=description,
             is_test=is_test,
             experiment_type=experiment_type,
+            properties=properties,
         )
 
     def add_trial_type(self, trial_type: str, runner: Runner) -> "MultiTypeExperiment":
@@ -164,6 +167,8 @@ class MultiTypeExperiment(Experiment):
             self._metric_to_canonical_name[metric.name] = canonical_name
         return self
 
+    # pyre-fixme[56]: While applying decorator
+    #  `ax.utils.common.docutils.copy_doc(...)`: Argument `metric_name` expected.
     @copy_doc(Experiment.remove_tracking_metric)
     def remove_tracking_metric(self, metric_name: str) -> "MultiTypeExperiment":
         if metric_name not in self._tracking_metrics:
@@ -178,6 +183,10 @@ class MultiTypeExperiment(Experiment):
             del self._metric_to_canonical_name[metric_name]
         return self
 
+    # pyre-fixme[56]: While applying decorator
+    #  `ax.utils.common.docutils.copy_doc(...)`: Expected `Experiment` for 1st param
+    #  but got `(self: MultiTypeExperiment, metrics:
+    #  Optional[List[ax.core.metric.Metric]] = ..., **(Any)) -> Data`.
     @copy_doc(Experiment.fetch_data)
     def fetch_data(self, metrics: Optional[List[Metric]] = None, **kwargs: Any) -> Data:
         return Data.from_multiple_data(
@@ -219,6 +228,7 @@ class MultiTypeExperiment(Experiment):
         """
         opt_config_types = {
             metric_name: self.default_trial_type
+            # pyre-fixme[16]: `Optional` has no attribute `metrics`.
             for metric_name in self.optimization_config.metrics.keys()
         }
         return {**opt_config_types, **self._metric_to_trial_type}
