@@ -122,7 +122,6 @@ def choose_generation_strategy(
     no_bayesian_optimization: bool = False,
     num_trials: Optional[int] = None,
     num_initialization_trials: Optional[int] = None,
-    no_max_parallelism: bool = False,
     max_parallelism_cap: Optional[int] = None,
     max_parallelism_override: Optional[int] = None,
 ) -> GenerationStrategy:
@@ -177,7 +176,7 @@ def choose_generation_strategy(
     if not no_bayesian_optimization and _should_use_gp(
         search_space=search_space, num_trials=num_trials
     ):
-        if not enforce_sequential_optimization and (
+        if not enforce_sequential_optimization and (  # pragma: no cover
             max_parallelism_override or max_parallelism_cap
         ):
             logger.info(
@@ -237,5 +236,5 @@ def choose_generation_strategy(
         )
         return gs
 
-    logger.info(f"Using Sobol generation strategy.")
+    logger.info("Using Sobol generation strategy.")
     return GenerationStrategy(steps=[_make_sobol_step(seed=random_seed)])
