@@ -21,7 +21,7 @@ if TYPE_CHECKING:
     from ax.modelbridge import base as base_modelbridge  # noqa F401  # pragma: no cover
 
 
-logger = get_logger("StandardizeY")
+logger = get_logger(__name__)
 
 
 class StandardizeY(Transform):
@@ -108,4 +108,8 @@ def compute_standardization_parameters(
         if s < 1e-8:
             Ystd[k] = 1.0
             logger.info(f"Outcome {k} is constant, within tolerance.")
+    # pyre-fixme[7]: Expected `Tuple[Dict[Union[Tuple[str, str], str], float],
+    #  Dict[Union[Tuple[str, str], str], float]]` but got `Tuple[Dict[Union[Tuple[str,
+    #  Union[None, bool, float, int, str]], str], typing.Any], Dict[Union[Tuple[str,
+    #  Union[None, bool, float, int, str]], str], typing.Any]]`.
     return Ymean, Ystd
