@@ -18,6 +18,7 @@ from ax.core.batch_trial import BatchTrial
 from ax.core.data import Data
 from ax.core.experiment import Experiment
 from ax.core.generator_run import GeneratorRun
+from ax.core.metric import Metric
 from ax.core.trial import Trial
 from ax.core.types import (
     TEvaluationOutcome,
@@ -39,13 +40,14 @@ from ax.service.utils.instantiation import (
     make_experiment,
     raw_data_to_evaluation,
 )
+from ax.service.utils.with_db_settings_base import DBSettings, WithDBSettingsBase
 from ax.storage.json_store.decoder import (
     generation_strategy_from_json,
     object_from_json,
 )
 from ax.storage.json_store.encoder import object_to_json
 from ax.utils.common.docutils import copy_doc
-from ax.utils.common.exec_utils import retry_on_exception
+from ax.utils.common.executils import retry_on_exception
 from ax.utils.common.logger import _round_floats_for_logging, get_logger
 from ax.utils.common.typeutils import (
     checked_cast,
@@ -59,26 +61,16 @@ from botorch.utils.sampling import manual_seed
 logger = get_logger(__name__)
 
 
-try:  # We don't require SQLAlchemy by default.
-    from sqlalchemy.exc import OperationalError
-    from ax.storage.sqa_store.structs import DBSettings
-    from ax.service.utils.storage import (
-        load_experiment_and_generation_strategy,
-        save_experiment,
-        save_generation_strategy,
-        save_new_trial,
-        save_updated_trial,
-    )
-
-    retry_exception_type_tuple = (OperationalError,)
-except ModuleNotFoundError:  # pragma: no cover
-    DBSettings = None
-    # Return a list of suppressable exceptions not including unimportable ones.
-    # pyre-fixme[9]: declared as `Tuple[...[OperationalError]]` but used as `Tuple[]`.
-    retry_exception_type_tuple = ()
+CHOLESKY_ERROR_ANNOTATION = (
+    "Cholesky errors typically occur when the same or very similar "
+    "arms are suggested repeatedly. This can mean the model has "
+    "already converged and you should avoid running further trials. "
+    "It will also help to convert integer or categorical parameters "
+    "to float ranges where reasonable.\nOriginal error: "
+)
 
 
-class AxClient:
+class AxClient(WithDBSettingsBase):
     """
     Convenience handler for management of experimentation cycle through a
     service-like API. External system manages scheduling of the cycle and makes
@@ -137,12 +129,13 @@ class AxClient:
     def __init__(
         self,
         generation_strategy: Optional[GenerationStrategy] = None,
-        db_settings: Any = None,
+        db_settings: Optional[DBSettings] = None,
         enforce_sequential_optimization: bool = True,
         random_seed: Optional[int] = None,
         verbose_logging: bool = True,
         suppress_storage_errors: bool = False,
     ) -> None:
+        super().__init__(db_settings=db_settings)
         if not verbose_logging:
             logger.setLevel(logging.WARNING)  # pragma: no cover
         else:
@@ -152,13 +145,6 @@ class AxClient:
                 "values in the logs are rounded to 2 decimal points."
             )
         self._generation_strategy = generation_strategy
-        if db_settings and (not DBSettings or not isinstance(db_settings, DBSettings)):
-            raise ValueError(
-                "`db_settings` argument should be of type ax.storage.sqa_store."
-                "structs.DBSettings. To use `DBSettings`, you will need SQLAlchemy "
-                "installed in your environment (can be installed through pip)."
-            )
-        self.db_settings = db_settings
         self._experiment: Optional[Experiment] = None
         self._enforce_sequential_optimization = enforce_sequential_optimization
         self._random_seed = random_seed
@@ -192,12 +178,23 @@ class AxClient:
 
         Args:
             parameters: List of dictionaries representing parameters in the
-                experiment search space. Required elements in the dictionaries
-                are: "name" (name of this parameter, string), "type" (type of the
-                parameter: "range", "fixed", or "choice", string), and "bounds"
-                for range parameters (list of two values, lower bound first),
-                "values" for choice parameters (list of values), and "value" for
-                fixed parameters (single value).
+                experiment search space.
+                Required elements in the dictionaries are:
+                1. "name" (name of parameter, string),
+                2. "type" (type of parameter: "range", "fixed", or "choice", string),
+                and one of the following:
+                3a. "bounds" for range parameters (list of two values, lower bound
+                first),
+                3b. "values" for choice parameters (list of values), or
+                3c. "value" for fixed parameters (single value).
+                Optional elements are:
+                1. "log_scale" (for float-valued range parameters, bool),
+                2. "value_type" (to specify type that values of this parameter should
+                take; expects "float", "int", "bool" or "str"),
+                3. "is_fidelity" (bool) and "target_value" (float) for fidelity
+                parameters,
+                4. "is_ordered" (bool) for choice parameters, and
+                5. "is_task" (bool) for task parameters.
             objective: Name of the metric used as objective in this experiment.
                 This metric must be present in `raw_data` argument to `complete_trial`.
             name: Name of the experiment to be created.
@@ -222,19 +219,15 @@ class AxClient:
                 `choose_generation_strategy` function which determines what
                 generation strategy should be used when none was specified on init.
         """
-        if self.db_settings and not name:
+        if self.db_settings_set and not name:
             raise ValueError(  # pragma: no cover
                 "Must give the experiment a name if `db_settings` is not None."
             )
-        if self.db_settings:
-            existing = None
-            try:
-                existing, _ = load_experiment_and_generation_strategy(
-                    experiment_name=not_none(name), db_settings=self.db_settings
-                )
-            except ValueError:  # Experiment does not exist, nothing to do.
-                pass
-            if existing:
+        if self.db_settings_set:
+            experiment_id, _ = self._get_experiment_and_generation_strategy_db_id(
+                experiment_name=not_none(name)
+            )
+            if experiment_id:
                 raise ValueError(
                     f"Experiment {name} already exists in the database. "
                     "To protect experiments that are running in production, "
@@ -254,7 +247,7 @@ class AxClient:
                 self._generation_strategy = None
             else:
                 raise ValueError(
-                    f"Experiment already created for this client instance. "
+                    "Experiment already created for this client instance. "
                     "Set the `overwrite_existing_experiment` to `True` to overwrite "
                     "with new experiment."
                 )
@@ -269,39 +262,81 @@ class AxClient:
             status_quo=status_quo,
             experiment_type=experiment_type,
         )
+
+        try:
+            self._save_experiment_to_db_if_possible(
+                experiment=self.experiment,
+                suppress_all_errors=self._suppress_storage_errors,
+            )
+        except Exception:
+            # Unset the experiment on this `AxClient` instance if encountered and
+            # raising an error from saving the experiment, to avoid a case where
+            # overall `create_experiment` call fails with a storage error, but
+            # `self._experiment` is still set and user has to specify the
+            # `ooverwrite_existing_experiment` kwarg to re-attempt exp. creation.
+            self._experiment = None
+            raise
+
         self._set_generation_strategy(
             choose_generation_strategy_kwargs=choose_generation_strategy_kwargs
         )
-        self._save_experiment_to_db_if_possible(
-            suppress_all_errors=self._suppress_storage_errors
-        )
         self._save_generation_strategy_to_db_if_possible(
-            suppress_all_errors=self._suppress_storage_errors
+            generation_strategy=self.generation_strategy,
+            suppress_all_errors=self._suppress_storage_errors,
         )
 
-    def get_next_trial(self) -> Tuple[TParameterization, int]:
+    @retry_on_exception(
+        logger=logger,
+        exception_types=(RuntimeError,),
+        suppress_all_errors=False,
+        wrap_error_message_in=CHOLESKY_ERROR_ANNOTATION,
+    )
+    def get_next_trial(
+        self, ttl_seconds: Optional[int] = None
+    ) -> Tuple[TParameterization, int]:
         """
         Generate trial with the next set of parameters to try in the iteration process.
 
         Note: Service API currently supports only 1-arm trials.
 
+        Args:
+            ttl_seconds: If specified, will consider the trial failed after this
+                many seconds. Used to detect dead trials that were not marked
+                failed properly.
+
         Returns:
             Tuple of trial parameterization, trial index
         """
-        trial = self.experiment.new_trial(generator_run=self._gen_new_generator_run())
+        trial = self.experiment.new_trial(
+            generator_run=self._gen_new_generator_run(), ttl_seconds=ttl_seconds
+        )
+
         logger.info(
             f"Generated new trial {trial.index} with parameters "
             f"{_round_floats_for_logging(item=not_none(trial.arm).parameters)}."
         )
         trial.mark_running(no_runner_required=True)
         self._save_new_trial_to_db_if_possible(
-            trial=trial, suppress_all_errors=self._suppress_storage_errors
+            experiment=self.experiment,
+            trial=trial,
+            suppress_all_errors=self._suppress_storage_errors,
         )
-        self._save_generation_strategy_to_db_if_possible(
-            suppress_all_errors=self._suppress_storage_errors
+        self._update_generation_strategy_in_db_if_possible(
+            generation_strategy=self.generation_strategy,
+            new_generator_runs=trial.generator_runs,
+            suppress_all_errors=self._suppress_storage_errors,
         )
         return not_none(trial.arm).parameters, trial.index
 
+    def abandon_trial(self, trial_index: int, reason: Optional[str] = None) -> None:
+        """Abandons a trial and adds optional metadata to it.
+
+        Args:
+            trial_index: Index of trial within the experiment.
+        """
+        trial = self._get_trial(trial_index=trial_index)
+        trial.mark_abandoned(reason=reason)
+
     def complete_trial(
         self,
         trial_index: int,
@@ -336,6 +371,13 @@ class AxClient:
             trial=trial, raw_data=raw_data, metadata=metadata, sample_sizes=sample_sizes
         )
         trial._run_metadata = metadata or {}
+        for metric_name in data.df["metric_name"].values:
+            if metric_name not in self.experiment.metrics:
+                logger.info(
+                    f"Data was logged for metric {metric_name} that was not yet "
+                    "tracked on the experiment. Adding it as tracking metric."
+                )
+                self.experiment.add_tracking_metric(Metric(name=metric_name))
         self.experiment.attach_data(data=data)
         trial.mark_completed()
         data_for_logging = _round_floats_for_logging(
@@ -346,7 +388,9 @@ class AxClient:
             f"{_round_floats_for_logging(item=data_for_logging)}."
         )
         self._save_updated_trial_to_db_if_possible(
-            trial=trial, suppress_all_errors=self._suppress_storage_errors
+            experiment=self.experiment,
+            trial=trial,
+            suppress_all_errors=self._suppress_storage_errors,
         )
 
     def update_trial_data(
@@ -386,6 +430,14 @@ class AxClient:
             trial=trial, raw_data=raw_data, metadata=metadata, sample_sizes=sample_sizes
         )
         trial._run_metadata.update(metadata or {})
+        # Registering trial data update is needed for generation strategies that
+        # leverage the `update` functionality of model and bridge setup and therefore
+        # need to be aware of new data added to experiment. Usually this happends
+        # seamlessly, by looking at newly completed trials, but in this case trial
+        # status does not change, so we manually register the new data.
+        # Currently this call will only result in a `NotImplementedError` if generation
+        # strategy uses `update` (`GenerationStep.use_update` is False by default).
+        self.generation_strategy._register_trial_data_update(trial=trial, data=data)
         self.experiment.attach_data(data, combine_with_last_data=True)
         data_for_logging = _round_floats_for_logging(
             item=evaluations[next(iter(evaluations.keys()))]
@@ -395,7 +447,8 @@ class AxClient:
             f"to trial {trial.index}."
         )
         self._save_experiment_to_db_if_possible(
-            suppress_all_errors=self._suppress_storage_errors
+            experiment=self.experiment,
+            suppress_all_errors=self._suppress_storage_errors,
         )
 
     def log_trial_failure(
@@ -413,29 +466,37 @@ class AxClient:
         if metadata is not None:
             trial._run_metadata = metadata
         self._save_experiment_to_db_if_possible(
-            suppress_all_errors=self._suppress_storage_errors
+            experiment=self.experiment,
+            suppress_all_errors=self._suppress_storage_errors,
         )
 
     def attach_trial(
-        self, parameters: TParameterization
+        self, parameters: TParameterization, ttl_seconds: Optional[int] = None
     ) -> Tuple[TParameterization, int]:
         """Attach a new trial with the given parameterization to the experiment.
 
         Args:
             parameters: Parameterization of the new trial.
+            ttl_seconds: If specified, will consider the trial failed after this
+                many seconds. Used to detect dead trials that were not marked
+                failed properly.
 
         Returns:
             Tuple of parameterization and trial index from newly created trial.
         """
         self._validate_search_space_membership(parameters=parameters)
-        trial = self.experiment.new_trial().add_arm(Arm(parameters=parameters))
+        trial = self.experiment.new_trial(ttl_seconds=ttl_seconds).add_arm(
+            Arm(parameters=parameters)
+        )
         trial.mark_running(no_runner_required=True)
         logger.info(
             "Attached custom parameterization "
             f"{_round_floats_for_logging(item=parameters)} as trial {trial.index}."
         )
         self._save_new_trial_to_db_if_possible(
-            trial=trial, suppress_all_errors=self._suppress_storage_errors
+            experiment=self.experiment,
+            trial=trial,
+            suppress_all_errors=self._suppress_storage_errors,
         )
         return not_none(trial.arm).parameters, trial.index
 
@@ -443,6 +504,11 @@ class AxClient:
         """Retrieve the parameterization of the trial by the given index."""
         return not_none(self._get_trial(trial_index).arm).parameters
 
+    # pyre-fixme[56]: While applying decorator
+    #  `ax.utils.common.docutils.copy_doc(...)`: Expected `Experiment` for 1st param
+    #  but got `(self: AxClient) -> Optional[Tuple[Dict[str, typing.Union[None, bool,
+    #  float, int, str]], Optional[Tuple[Dict[str, float], Optional[Dict[str,
+    #  typing.Dict[str, float]]]]]]]`.
     @copy_doc(best_point_utils.get_best_parameters)
     def get_best_parameters(
         self,
@@ -497,18 +563,21 @@ class AxClient:
         """
         if not self.experiment.trials:
             raise ValueError("Cannot generate plot as there are no trials.")
+        # pyre-fixme[16]: `Optional` has no attribute `objective`.
         objective_name = self.experiment.optimization_config.objective.metric.name
         best_objectives = np.array(
             [
                 [
                     checked_cast(Trial, trial).objective_mean
                     for trial in self.experiment.trials.values()
+                    if trial.status.is_completed
                 ]
             ]
         )
         hover_labels = [
             _format_dict(not_none(checked_cast(Trial, trial).arm).parameters)
             for trial in self.experiment.trials.values()
+            if trial.status.is_completed
         ]
         return optimization_trace_single_method(
             y=(
@@ -650,20 +719,21 @@ class AxClient:
         Returns:
             Experiment object.
         """
-        if not self.db_settings:
-            raise ValueError(  # pragma: no cover
-                "Cannot load an experiment in the absence of the DB settings."
-                "Please initialize `AxClient` with DBSettings."
-            )
-        experiment, generation_strategy = load_experiment_and_generation_strategy(
-            experiment_name=experiment_name, db_settings=self.db_settings
+        experiment, generation_strategy = self._load_experiment_and_generation_strategy(
+            experiment_name=experiment_name
         )
+        if experiment is None:
+            raise ValueError(f"Experiment by name '{experiment_name}' not found.")
         self._experiment = experiment
         logger.info(f"Loaded {experiment}.")
         if generation_strategy is None:  # pragma: no cover
             self._set_generation_strategy(
                 choose_generation_strategy_kwargs=choose_generation_strategy_kwargs
             )
+            self._save_generation_strategy_to_db_if_possible(
+                generation_strategy=self.generation_strategy,
+                suppress_all_errors=self._suppress_storage_errors,
+            )
         else:
             self._generation_strategy = generation_strategy
             logger.info(
@@ -823,95 +893,12 @@ class AxClient:
         )
         if self._generation_strategy is None:
             self._generation_strategy = choose_generation_strategy(
-                search_space=not_none(self._experiment).search_space,
+                search_space=self.experiment.search_space,
                 enforce_sequential_optimization=enforce_sequential_optimization,
                 random_seed=random_seed,
                 **choose_generation_strategy_kwargs,
             )
 
-    @retry_on_exception(
-        retries=3,
-        default_return_on_suppression=False,
-        exception_type=retry_exception_type_tuple,
-    )
-    def _save_experiment_to_db_if_possible(
-        self, suppress_all_errors: bool = False
-    ) -> bool:
-        """Saves attached experiment and generation strategy if DB settings are
-        set on this AxClient instance.
-
-        Returns:
-            bool: Whether the experiment was saved.
-        """
-        if self.db_settings is not None:
-            save_experiment(experiment=self.experiment, db_settings=self.db_settings)
-            return True
-        return False
-
-    @retry_on_exception(
-        retries=3,
-        default_return_on_suppression=False,
-        exception_type=retry_exception_type_tuple,
-    )
-    def _save_new_trial_to_db_if_possible(
-        self, trial: BaseTrial, suppress_all_errors: bool = False
-    ) -> bool:
-        """Saves given trial and generation strategy if DB settings are
-        set on this AxClient instance.
-
-        Returns:
-            bool: Whether the experiment was saved.
-        """
-        if self.db_settings is not None:
-            save_new_trial(
-                experiment=self.experiment, trial=trial, db_settings=self.db_settings
-            )
-            return True
-        return False
-
-    @retry_on_exception(
-        retries=3,
-        default_return_on_suppression=False,
-        exception_type=retry_exception_type_tuple,
-    )
-    def _save_updated_trial_to_db_if_possible(
-        self, trial: BaseTrial, suppress_all_errors: bool = False
-    ) -> bool:
-        """Saves attached experiment and generation strategy if DB settings are
-        set on this AxClient instance.
-
-        Returns:
-            bool: Whether the experiment was saved.
-        """
-        if self.db_settings is not None:
-            save_updated_trial(
-                experiment=self.experiment, trial=trial, db_settings=self.db_settings
-            )
-            return True
-        return False
-
-    @retry_on_exception(
-        retries=3,
-        default_return_on_suppression=False,
-        exception_type=retry_exception_type_tuple,
-    )
-    def _save_generation_strategy_to_db_if_possible(
-        self, suppress_all_errors: bool = False
-    ) -> bool:
-        """Saves attached experiment and generation strategy if DB settings are
-        set on this AxClient instance.
-
-        Returns:
-            bool: Whether the experiment was saved.
-        """
-        if self.db_settings is not None:
-            save_generation_strategy(
-                generation_strategy=self.generation_strategy,
-                db_settings=self.db_settings,
-            )
-            return True
-        return False
-
     def _gen_new_generator_run(self, n: int = 1) -> GeneratorRun:
         """Generate new generator run for this experiment.
 
@@ -981,13 +968,18 @@ class AxClient:
         if isinstance(trial, BatchTrial):
             assert isinstance(  # pragma: no cover
                 raw_data, dict
-            ), f"Raw data must be a dict for batched trials."
+            ), "Raw data must be a dict for batched trials."
         elif isinstance(trial, Trial):
             arm_name = not_none(trial.arm).name
             raw_data = {arm_name: raw_data}  # pyre-ignore[9]
         else:  # pragma: no cover
             raise ValueError(f"Unexpected trial type: {type(trial)}.")
         assert isinstance(raw_data, dict)
+        not_trial_arm_names = set(raw_data.keys()) - set(trial.arms_by_name.keys())
+        if not_trial_arm_names:
+            raise ValueError(
+                f"Arms {not_trial_arm_names} are not part of trial #{trial.index}."
+            )
         evaluations = {
             arm_name: raw_data_to_evaluation(
                 raw_data=raw_data[arm_name], objective_name=self.objective_name
@@ -1040,7 +1032,7 @@ class AxClient:
                     f"Value for parameter {p_name} is of type {typ}, expected "
                     f"{parameter.python_type}. If the intention was to have the "
                     f"parameter on experiment be of type {typ}, set `value_type` "
-                    "on experiment creation for {p_name}."
+                    f"on experiment creation for {p_name}."
                 )
 
     # -------- Backward-compatibility with old save / load method names. -------
